#!/usr/bin/python3

#
# Copyright (C) 2013 Felix Geyer <debfx@fobos.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or (at your option)
# version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Parses keysymdef.h to construct a unicode symbol -> keysym mapping table.
#
# The lines that are parsed look like this:
# #define XK_Aogonek 0x01a1  /* U+0104 LATIN CAPITAL LETTER A WITH OGONEK */
#
# This would create a 0x0104 -> 0x01a1 mapping.
#

import sys
import re
import collections

cols = 8

if len(sys.argv) >= 2:
    keysymdef = sys.argv[1]
else:
    keysymdef = "/usr/include/X11/keysymdef.h"

keysymMap = {}

with open(keysymdef, "r") as fid:
    for line in fid:
        match = re.search(r'0x([0-9a-fA-F]+)\s+/\* U\+([0-9a-fA-F]+)', line)
        if match:
            keysym = int(match.group(1), 16)
            unicodeVal = int(match.group(2), 16)

            # ignore 1:1 mappings
            if 0x0020 <= keysym <= 0x007e or 0x00a0 <= keysym <= 0x00ff:
                continue
            # ignore unicode | 0x01000000 mappings
            elif keysym >= 0x1000000:
                continue

            keysymMap[unicodeVal] = keysym

keysymMap = collections.OrderedDict(sorted(keysymMap.items(), key=lambda t: t[0]))

print("""/*
 *  Automatically generated by keysymmap.py from parsing keysymdef.h.
 */
""")

print("const int AutoTypePlatformX11::m_unicodeToKeysymLen = {0};".format(len(keysymMap)))

print()

print("const uint AutoTypePlatformX11::m_unicodeToKeysymKeys[] = {")
keys = keysymMap.keys()
keyLen = len(keys)
for idx, val in enumerate(keys, start=1):
    hexVal = "{0:#0{1}x}".format(val, 6)

    if idx == keyLen:
        print(hexVal)
    elif (idx % cols) == 0:
        print(hexVal + ",")
    elif ((idx - 1) % cols) == 0:
        print("    " + hexVal + ", ", end="")
    else:
        print(hexVal + ", ", end="")
print("};")

print()

print("const uint AutoTypePlatformX11::m_unicodeToKeysymValues[] = {")
values = keysymMap.values()
valuesLen = len(values)
for idx, val in enumerate(values, start=1):
    hexVal = "{0:#0{1}x}".format(val, 6)

    if idx == valuesLen:
        print(hexVal)
    elif (idx % cols) == 0:
        print(hexVal + ",")
    elif ((idx - 1) % cols) == 0:
        print("    " + hexVal + ", ", end="")
    else:
        print(hexVal + ", ", end="")
print("};")



#!/usr/bin/env python3
import json
import os

# Download Transifex languages dump at: https://www.transifex.com/api/2/project/keepassxc/languages
# Language information from https://www.wikiwand.com/en/List_of_ISO_639-1_codes and http://www.lingoes.net/en/translator/langcode.htm

LANGS = {
    "ar" : "العربية (Arabic)",
    "bn" : "বাংলা (Bengali)",
    "ca" : "català (Catalan)",
    "cs" : "čeština (Czech)",
    "da" : "dansk (Danish)",
    "de" : "Deutsch (German)",
    "el" : "ελληνικά (Greek)",
    "en_GB" : "English (UK)",
    "en_US" : "English (USA)",
    "eo" : "Esperanto (Esperanto)",
    "es" : "Español (Spanish)",
    "et" : "eesti (Estonian)",
    "eu" : "euskara (Basque)",
    "fa" : "فارسی (Farsi)",
    "fa_IR" : "فارسی (Farsi (Iran))",
    "fi" : "suomi (Finnish)",
    "fr" : "français (French)",
    "gl" : "Galego (Galician)",
    "he" : "עברית (Hebrew)",
    "hr_HR" : "hrvatski jezik (Croatian)",
    "hu" : "magyar (Hungarian)",
    "id" : "Bahasa (Indonesian)",
    "is_IS" : "Íslenska (Icelandic)",
    "it" : "Italiano (Italian)",
    "ja" : "日本語 (Japanese)",
    "kk" : "қазақ тілі (Kazakh)",
    "ko" : "한국어 (Korean)",
    "la" : "latine (Latin)",
    "lt" : "lietuvių kalba (Lithuanian)",
    "lv" : "latviešu valoda (Latvian)",
    "nb" : "Norsk Bokmål (Norwegian Bokmål)",
    "nl_NL" : "Nederlands (Dutch)",
    "my" : "ဗမာစာ (Burmese)",
    "pa" : "ਪੰਜਾਬੀ (Punjabi)",
    "pa_IN" : "ਪੰਜਾਬੀ (Punjabi (India))",
    "pl" : "język polski (Polish)",
    "pt" : "Português (Portuguese)",
    "pt_BR" : "Português (Portuguese (Brazil))",
    "pt_PT" : "Português (Portuguese (Portugal))",
    "ro" : "Română (Romanian)",
    "ru" : "русский (Russian)",
    "sk" : "Slovenčina (Slovak)",
    "sl_SI" : "Slovenščina (Slovenian)",
    "sr" : "српски језик (Serbian)",
    "sv" : "Svenska (Swedish)",
    "th" : "ไทย (Thai)",
    "tr" : "Türkçe (Turkish)",
    "uk" : "Українська (Ukrainian)",
    "zh_CN" : "中文 (Chinese (Simplified))",
    "zh_TW" : "中文 (台灣) (Chinese (Traditional))",
}

TEMPLATE = "<li><strong>{0}</strong>: {1}</li>\n"

if not os.path.exists("languages.json"):
    print("Could not find 'languages.json' in current directory!")
    print("Save the output from https://www.transifex.com/api/2/project/keepassxc/languages")
    exit(0)

with open("languages.json") as json_file:
    output = open("translators.html", "w", encoding="utf-8")
    languages = json.load(json_file)
    for lang in languages:
        code = lang["language_code"]
        if code not in LANGS:
            print("WARNING: Could not find language code:", code)
            continue
        translators = ", ".join(sorted(lang["reviewers"] + lang["translators"], key=str.casefold))
        output.write(TEMPLATE.format(LANGS[code], translators))
    output.close()
    print("Language translators written to 'translators.html'!")